﻿using Ecng.Common;
using Ecng.Xaml;
using StockSharp.Algo;
using StockSharp.Algo.Candles;
using StockSharp.BusinessEntities;
using StockSharp.Logging;
using StockSharp.Messages;
using StockSharp.Quik;
using StockSharp.Xaml.Charting;
using System;
using System.Collections.Generic;
using System.Net;
using System.Security;
using System.Windows;
using Ecng.Configuration;
using System.Linq;
using System.Diagnostics;

namespace WpfApplication1
{
    /// <summary>
    /// Логика взаимодействия для MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, ILogReceiver
    {
        private QuikTrader _qTrader;
        private object _adress = "127.0.0.1:5002";
        private object _pas = "quik";
        private string _login = "quik";

        private CandleManager _candles1;
        private CandleManager _candles2;
        private ChartArea _chartArea1 = new ChartArea();
        private ChartArea _chartArea2 = new ChartArea();
        private CandleSeries _candleSeries1;
        private CandleSeries _candleSeries2;
        private ChartCandleElement _candlesElem1;
        private ChartCandleElement _candlesElem2;

        private WeightedIndexSecurity _index1;
        private WeightedIndexSecurity _index2;


        private const string _secCode1 = "GZU7";
        private const string _secCode2 = "HYU7";
        private const string _secCode3 = "SRU7";

        private Security _sec1;
        private Security _sec2;
        private Security _sec3;


        private bool _started = false;

        public MainWindow()
        {
            InitializeComponent();

            _qTrader = ConnectToQuik();

            _candles1 = new CandleManager((Connector)_qTrader);
            _candles2 = new CandleManager(_qTrader);

            _candles1.Processing += DrawCandles1;
            _candles2.Processing += DrawCandles2;

            _candlesElem1 = new ChartCandleElement();
            _candlesElem2 = new ChartCandleElement();

            Plot1.Areas.Add(_chartArea1);
            Plot2.Areas.Add(_chartArea2);

            _chartArea1.Elements.Add(_candlesElem1);
            _chartArea2.Elements.Add(_candlesElem2);




            _qTrader.LookupSecuritiesResult += (e, _sec) => this.GuiAsync(() =>
            {
                if (_started == false)
                {
                    _sec1 = _qTrader.Securities.FirstOrDefault(s => s.Code == _secCode1);
                    _sec2 = _qTrader.Securities.FirstOrDefault(s => s.Code == _secCode2);
                    _sec3 = _qTrader.Securities.FirstOrDefault(s => s.Code == _secCode3);

                    if (_sec1 != null && _sec2 != null && _sec3 != null)
                    {
                        _index1 = new WeightedIndexSecurity() { Board = ExchangeBoard.Forts, Id = "Index1" };

                        _index1.Weights.Add(new KeyValuePair<SecurityId, decimal>(_sec1.ToSecurityId(), 1));
                        _index1.Weights.Add(new KeyValuePair<SecurityId, decimal>(_sec2.ToSecurityId(), -1));


                        _candleSeries1 = new CandleSeries(typeof(TimeFrameCandle), _index1, TimeSpan.FromMinutes(1)) { BuildCandlesMode = BuildCandlesModes.Load };

                        ConfigManager.RegisterService<ISecurityProvider>(_qTrader);


                        _candles1.Start(_candleSeries1);



                        _index2 = new WeightedIndexSecurity() { Board = ExchangeBoard.Forts, Id = "Index2" };

                        _index2.Weights.Add(new KeyValuePair<SecurityId, decimal>(_sec1.ToSecurityId(), 1));
                        _index2.Weights.Add(new KeyValuePair<SecurityId, decimal>(_sec3.ToSecurityId(), -1));


                        _candleSeries2 = new CandleSeries(typeof(TimeFrameCandle), _index2, TimeSpan.FromMinutes(1)) { BuildCandlesMode = BuildCandlesModes.Load };

                        ConfigManager.RegisterService<ISecurityProvider>(_qTrader);


                        _candles2.Start(_candleSeries2);

                        _started = true;
                    }
                }

            });


            _qTrader.Connect();
        }

        public QuikTrader ConnectToQuik()
        {
            try
            {
                if (_qTrader == null)
                {
                    // создаем подключение
                    _qTrader = new QuikTrader()
                    {
                        LuaLogin = _login,
                        LuaFixServerAddress = _adress.To<EndPoint>(),
                        LuaPassword = _pas.To<SecureString>()
                    };

                    _qTrader.Restored += () => this.GuiAsync(() =>
                    {
                        //this.AddInfoLog("Quik. Подключение восстановлено.");
                    });

                    _qTrader.Connected += () => this.GuiAsync(() =>
                    {
                        //this.AddInfoLog("Quik. Установлено подключение.");
                    });

                    _qTrader.Disconnected += () => this.GuiAsync(() =>
                    {
                        //this.AddInfoLog("Quik. Подключение разорвано.");
                    });

                    _qTrader.ConnectionError += error => this.GuiAsync(() =>
                    {
                        //this.AddErrorLog("Quik. Ошибка подключения. " + error.ToString());
                    });

                    _qTrader.Error += error => this.GuiAsync(() =>
                    {
                        //this.AddErrorLog("Quik. Ошибка обработки данных. " + error.ToString());
                    });

                    _qTrader.MarketDataSubscriptionFailed += (security, type, error) => this.GuiAsync(() =>
                    {
                        //this.AddErrorLog("Quik. Ошибка подписки на {0} для {1}.", type, security);
                    });

                }

                return _qTrader;
            }
            catch (Exception ex)
            {
                //this.AddErrorLog("Quik. Ошибка подключения: {0}", ex.Message);
                return null;
            }
        }


        private void DrawCandles1(CandleSeries series, Candle candle)
        {
            Debug.WriteLine(string.Format("series= {0}, candle= {1}, candleseries= {2}", series.Security.Id, candle.Security.Id, _candleSeries1.Security.Id));

            if (series == _candleSeries1)
            {
                var data = new ChartDrawData();
                data.Group(candle.OpenTime).Add(_candlesElem1, candle);

                try
                {
                    Plot1.Draw(data);
                }
                catch
                { }
            }
        }

        private void DrawCandles2(CandleSeries series, Candle candle)
        {
            Debug.WriteLine(string.Format("series= {0}, candle= {1}, candleseries= {2}", series.Security.Id, candle.Security.Id, _candleSeries2.Security.Id));

            if (series == _candleSeries2)
            {
                var data = new ChartDrawData();
                data.Group(candle.OpenTime).Add(_candlesElem2, candle);

                try
                {
                    Plot2.Draw(data);
                }
                catch
                { }
            }
        }

        #region Implementation of ILogSource
        private readonly Guid _id = Guid.NewGuid();

        public Guid Id
        {
            get { return _id; }
        }

        string ILogSource.Name { get; }

        public LogLevels LogLevel
        {
            get
            {
                return LogLevels.Debug;
            }
            set { }
        }

        ILogSource ILogSource.Parent
        {
            get { return null; }
            set { }
        }

        public bool IsRoot
        {
            get
            {
                throw new NotImplementedException();
            }
        }

        public DateTimeOffset CurrentTime
        {
            get
            {
                return DateTimeOffset.Now;
            }
        }

        public event Action<LogMessage> Log;

        public void Dispose()
        {
            //throw new NotImplementedException();
        }

        public void AddLog(LogMessage message)
        {
            //throw new NotImplementedException();
            Log.Invoke(message);
        }



        #endregion
    }
}
